@@ -21,38 +21,38 @@ mtcars = modify(mtcars,{
 # row_labels = c("row_vars", "row_vars_values", "summary_vars", "fun_names", "stat_names"),
 # col_labels = c("col_vars", "col_vars_values")
 
-# table_summary(mtcars %except% c("cyl", "am"), col_vars = mtcars$am, fun = mean)
-# table_means(mtcars %except% c("cyl", "am"), col_vars = mtcars$am)
-# table_medians(mtcars %except% c("cyl", "am"), col_vars = mtcars$am)
-# table_sums(mtcars %except% c("cyl", "am"), col_vars = mtcars$am)
-# table_sums(mtcars %except% c("cyl", "am"), col_vars = mtcars$am, weight = 2)
-# expect_error(cro_fun(mtcars %except% c("cyl", "am"), col_vars = mtcars$am, 
+# table_summary(mtcars %>% except("cyl", "am"), col_vars = mtcars$am, fun = mean)
+# table_means(mtcars %>% except("cyl", "am"), col_vars = mtcars$am)
+# table_medians(mtcars %>% except("cyl", "am"), col_vars = mtcars$am)
+# table_sums(mtcars %>% except("cyl", "am"), col_vars = mtcars$am)
+# table_sums(mtcars %>% except("cyl", "am"), col_vars = mtcars$am, weight = 2)
+# expect_error(cro_fun(mtcars %>% except("cyl", "am"), col_vars = mtcars$am, 
 #                      fun = mean)
 #              )
-# expect_error(cro_fun(mtcars %except% c("cyl", "am"), col_vars = mtcars$am, fun = mean))
-# expect_error(cro_fun(mtcars %except% c("cyl", "am"), col_vars = mtcars$am, fun = mean,
+# expect_error(cro_fun(mtcars %>% except("cyl", "am"), col_vars = mtcars$am, fun = mean))
+# expect_error(cro_fun(mtcars %>% except("cyl", "am"), col_vars = mtcars$am, fun = mean,
 #               row_labels = c("stat_names", "col_vars_values", "row_vars", "row_vars_values", "summary_vars",  "col_vars"),
 #               col_labels = c("adasd", "rvtrvtt")
 # ))
-# expect_error(cro_fun(mtcars %except% c("cyl", "am"), col_vars = mtcars$am, fun = mean,
+# expect_error(cro_fun(mtcars %>% except("cyl", "am"), col_vars = mtcars$am, fun = mean,
 #               col_labels = c("stat_names", "col_vars_values", "row_vars", "row_vars_values", "summary_vars",  "col_vars"),
 #               row_labels = c("adasd", "rvtrvtt")
 # ))
 
 #######
 expect_error(
-    cro_fun(mtcars %except% c("cyl", "am"), col_vars = mtcars$am, fun = mean, weight = 2)
+    cro_fun(mtcars %>% except("cyl", "am"), col_vars = mtcars$am, fun = mean, weight = 2)
     )
 
 
 
 expect_equal_to_reference(
-    cro_fun(mtcars %except% c("cyl", "am"), col_vars = mtcars$am, fun = combine_functions(w_mean), weight = 2),
+    cro_fun(mtcars %>% except("cyl", "am"), col_vars = mtcars$am, fun = combine_functions(w_mean), weight = 2),
                           "rds/table_summary0.rds")
 
 
 expect_equal_to_reference(
-    cro_fun(mtcars %except% c("cyl", "am"), col_vars = list("Total", mtcars$am), 
+    cro_fun(mtcars %>% except("cyl", "am"), col_vars = list("Total", mtcars$am), 
             fun = combine_functions(mean)),
     "rds/table_summary1.rds"
     )
@@ -60,7 +60,7 @@ expect_equal_to_reference(
 
 add_val_lab(mtcars$am) = c(HardToSay = 3)
 expect_equal_to_reference(
-    cro_fun(mtcars %except% c("cyl", "am"), col_vars = list("Total", mtcars$am), 
+    cro_fun(mtcars %>% except("cyl", "am"), col_vars = list("Total", mtcars$am), 
             fun = combine_functions(mean))
     ,"rds/table_summary2.rds"
 )
@@ -84,19 +84,19 @@ mtcars = modify(mtcars,{
 
 add_val_lab(mtcars$am) = c(HardToSay = 3)
 expect_equal_to_reference(
-cro_fun(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+cro_fun(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
               fun = combine_functions(mean))
 ,"rds/table_summary3.rds"
 )
 
 expect_equal_to_reference(
-cro_fun(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+cro_fun(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
         fun = mean)
 ,"rds/table_summary4.rds"
 )
 
 expect_equal_to_reference(
-cro_fun_df(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+cro_fun_df(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
                fun = function(x) {
                    res = t(colMeans(x) )
                    rownames(res) = "mean"
@@ -106,7 +106,7 @@ cro_fun_df(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
 )
 
 # expect_equal_to_reference(
-# cro_fun(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+# cro_fun(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
 #               row_labels = c("row_vars", "row_vars_values", "stat_names"),
 #               col_labels = c("col_vars", "summary_vars", "col_vars_values"), fun = mean)
 # ,"rds/table_summary6.rds"
@@ -116,7 +116,7 @@ cro_fun_df(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
 ########## rowlabels
 
 # expect_equal_to_reference(
-# cro_fun(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+# cro_fun(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
 #               row_vars = mtcars$vs,
 #              fun = combine_functions(mean))
 # ,"rds/table_summary7.rds"
@@ -124,7 +124,7 @@ cro_fun_df(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
 
 
 # expect_equal_to_reference(
-# cro_fun(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+# cro_fun(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
 #               row_vars = mtcars$vs,
 #               row_labels = c("row_vars", "row_vars_values", "summary_vars"),
 #               col_labels = c("col_vars", "col_vars_values", "stat_names"), fun = mean)
@@ -132,7 +132,7 @@ cro_fun_df(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
 # )
 
 # expect_equal_to_reference(
-# cro_fun(mtcars %except% c("vs", "am"), 
+# cro_fun(mtcars %>% except("vs", "am"), 
 #               col_vars = mtcars$vs,
 #         row_vars = list("Total", mtcars$am), 
 #         fun = mean)
@@ -140,7 +140,7 @@ cro_fun_df(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
 # )
 
 # expect_equal_to_reference(
-# cro_fun(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+# cro_fun(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
 #               row_vars = mtcars$vs,
 #               row_labels = c("row_vars", "row_vars_values",  "col_vars_values", "summary_vars", "stat_names"),
 #               col_labels = c("col_vars"), fun = mean)
@@ -150,13 +150,13 @@ cro_fun_df(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
 add_val_lab(mtcars$vs) = c("Don't know" = 88)
 
 # expect_equal_to_reference(
-# cro_fun(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+# cro_fun(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
 #               row_vars = mtcars$vs, fun = combine_functions(mean))
 # ,"rds/table_summary11.rds"
 # )
 
 # expect_equal_to_reference(
-# cro_fun(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+# cro_fun(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
 #               row_vars = mtcars$vs,
 #               row_labels = c("row_vars", "row_vars_values", "summary_vars"),
 #               col_labels = c("col_vars", "col_vars_values", "stat_names"), fun = mean)
@@ -165,7 +165,7 @@ add_val_lab(mtcars$vs) = c("Don't know" = 88)
 
 
 # expect_equal_to_reference(
-# cro_fun(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+# cro_fun(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
 #               row_vars = mtcars$vs,
 #               row_labels = c("row_vars", "row_vars_values", "summary_vars", "stat_names"),
 #               col_labels = c("col_vars", "col_vars_values", "summary_vars"), fun = mean)
@@ -174,7 +174,7 @@ add_val_lab(mtcars$vs) = c("Don't know" = 88)
 
 
 expect_equal_to_reference(
-cro_fun_df(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+cro_fun_df(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
               row_vars = mtcars$vs, 
         fun = function(x) {
             res = t(colMeans(x) )
@@ -186,7 +186,7 @@ cro_fun_df(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
 
 
 # expect_equal_to_reference(
-# cro_fun(mtcars %except% c("vs", "am"), col_vars = list("Total", mtcars$am),
+# cro_fun(mtcars %>% except("vs", "am"), col_vars = list("Total", mtcars$am),
 #               row_vars = mtcars$vs,
 #               row_labels = c("row_vars", "row_vars_values",  "col_vars_values", "summary_vars", "stat_names"),
 #               col_labels = c("col_vars"), fun = mean)
@@ -303,3 +303,87 @@ expect_identical(
                       fun = combine_functions(w_mean)))
 )
 # calc(w, fre(a1_1 %to% a1_6))
+context("cro_mean_sd_n")
+
+data(mtcars)
+mtcars = modify(mtcars,{
+    var_lab(mpg) = "Miles/(US) gallon"
+    var_lab(cyl) = "Number of cylinders"
+    var_lab(disp) = "Displacement (cu.in.)"
+    var_lab(hp) = "Gross horsepower"
+    var_lab(drat) = "Rear axle ratio"
+    var_lab(wt) = "Weight (lb/1000)"
+    var_lab(qsec) = "1/4 mile time"
+    var_lab(vs) = "V/S"
+    val_lab(vs) = c("Straight" = 0, "V" = 1)
+    var_lab(am) = "Transmission (0 = automatic, 1 = manual)"
+    val_lab(am) = c(" automatic" = 0, " manual" =  1)
+    var_lab(gear) = "Number of forward gears"
+    var_lab(carb) = "Number of carburetors"
+})
+
+expect_equal_to_reference(
+    mtcars %calc% cro_mean_sd_n(list(mpg, disp, wt), list(total(), am, vs)),
+    "rds/cro_mean_sd_n1.rds"
+)
+expect_equal_to_reference(
+    mtcars %calc% cro_mean_sd_n(list(mpg, disp, wt), list(total(), am, vs), weight = 0.1,
+                                weighted_valid_n = TRUE),
+    "rds/cro_mean_sd_n2.rds"
+)
+
+expect_equal_to_reference(
+    mtcars %calc% cro_mean_sd_n(list(mpg, disp, wt), list(total(), am, vs), 
+                                labels = c("m", "s", "n"),
+                                weight = 0.1,
+                                weighted_valid_n = TRUE),
+    "rds/cro_mean_sd_n3.rds"
+)
+
+
+context("cro_fun unsafe labels")
+
+fun = function(x) c(sum(x), mean(x), length(x))
+
+fun2 = combine_functions(sum = sum, mean = mean, length = length)
+
+expect_identical(cro_fun(list(mtcars$mpg, mtcars$disp, mtcars$wt), 
+                         list(total(), mtcars$am, mtcars$vs), 
+                               fun = fun,
+                               unsafe = c("sum", "mean", "length")),
+                 cro_fun(list(mtcars$mpg, mtcars$disp, mtcars$wt), 
+                         list(total(), mtcars$am, mtcars$vs), 
+                         fun = fun2)
+)
+
+fun2 = combine_functions("1" = sum, "2" = mean, "3" = length)
+
+expect_identical(cro_fun(list(mtcars$mpg, mtcars$disp, mtcars$wt), 
+                         list(total(), mtcars$am, mtcars$vs), 
+                         fun = fun,
+                         unsafe = 1:3),
+                 cro_fun(list(mtcars$mpg, mtcars$disp, mtcars$wt), 
+                         list(total(), mtcars$am, mtcars$vs), 
+                         fun = fun2)
+)
+
+expect_error(cro_fun(list(mtcars$mpg, mtcars$disp, mtcars$wt), 
+                     list(total(), mtcars$am, mtcars$vs), 
+                     fun = fun,
+                     unsafe = c("sum", "mean", "length","ffaafa")))
+
+expect_error(cro_fun(list(mtcars$mpg, mtcars$disp, mtcars$wt), 
+                     list(total(), mtcars$am, mtcars$vs), 
+                     fun = fun,
+                     unsafe = c("sum", "length", "length")))
+
+
+#################
+
+expect_equal(
+cro_fun(c(1:3, NA, NA, NA), c(1,1,1,2,2,2), weight = 1.9, fun = w_sd),
+structure(list(row_labels = "c(1:3, NA, NA, NA)", `c(1, 1, 1, 2, 2, 2)|1` = 0.899172196132572, 
+`c(1, 1, 1, 2, 2, 2)|2` = NA_real_), .Names = c("row_labels", 
+"c(1, 1, 1, 2, 2, 2)|1", "c(1, 1, 1, 2, 2, 2)|2"), row.names = c(NA, 
+-1L), class = c("etable", "data.frame"))
+)
