@@ -8,6 +8,7 @@ expect_error(recode(1, ~ 42))
 
 expect_identical(recode(1:5, 1~-1), c(-1, NA, NA, NA, NA))
 expect_identical(recode(1:5, 1~-1, other ~ copy), c(-1, 2, 3, 4, 5))
+expect_identical(recode(1:5, 1~-1, other() ~ copy()), c(-1, 2, 3, 4, 5))
 b = 1:5
 recode(b) = 1~-1
 expect_identical(b, c(-1, 2, 3, 4, 5))
@@ -435,14 +436,15 @@ expect_identical(class(a), c("labelled", "character"))
 
 a = factor(letters[1:4])
 
-res = factor(c("z", "b", "c", "d"), levels = c("a", "b", "c", "d", "z"))
-b = recode(a, "a" ~ "z", other ~ copy)
+res = factor(c("a", "z", "c", "d"), levels = c("z", "a", "c", "d"))
+b = recode(a, "b" ~ "z", other ~ copy)
 expect_identical(b, res)
 
-b = recode(a, "a" ~ "z")
+res = factor(c("z", "b", "c", "d"), levels = c("a", "b", "c", "d", "z"))
+b = recode(a, "a" ~ factor("z"))
 res2 = res
 res2[2:4] = NA
-expect_identical(b, res2) 
+expect_identical(b, factor(res2)) 
 
 recode(a) = "a" ~ "z"
 expect_identical(a, res)
@@ -456,14 +458,29 @@ expect_identical(a, res)
 
 a = as.POSIXct("2016-10-01")
 
-b = recode(a, "2016-10-01" ~ "2016-10-02")
+b = recode(a, "2016-10-01" ~ as.POSIXct("2016-10-02"))
 
 expect_equal(b, as.POSIXct("2016-10-02"))
 
+b = recode(a, "2016-10-01" ~ "2016-10-02")
+
+expect_equal(b, "2016-10-02")
+
 recode(a) = c("2016-10-01" ~ "2016-10-02")
 
 expect_identical(a, as.POSIXct("2016-10-02"))
 
+a = as.POSIXct(c("2016-10-01", "2017-05-10"))
+res = as.POSIXct(c("2016-10-02", "2017-05-11"))
+b = recode(a, "2016-10-01" ~ as.POSIXct("2016-10-02"), "2017-05-10" ~ "2017-05-11")
+expect_equal(b, res)
+
+b = recode(a, "2016-10-01" ~ "2016-10-02", "2017-05-10" ~ "2017-05-11")
+expect_equal(b, as.character(res))
+
+recode(a) = "2016-10-01" ~ "2016-10-02"
+expect_identical(a, as.POSIXct(c("2016-10-02", "2017-05-10")))
+
 context("dot notation")
 
 a = 1:5
@@ -501,8 +518,7 @@ empty_iris[[1]] = NA
 empty_iris[[2]] = NA
 empty_iris[[3]] = NA
 empty_iris[[4]] = NA
-empty_iris[[5]] = iris$Species
-empty_iris[[5]][] = NA
+empty_iris[[5]] = NA
 rownames(empty_iris) = as.character(1:150)
 res = list(rep(NA, 4), rep(NA, 4), empty_iris)
 names(res) = c("a", "b", "c")
@@ -512,6 +528,72 @@ expect_identical(mk_emtpy_obj(ab), res)
 
 context("%into%")
 
+remove_if_exists = function(...){
+    for(each in unlist(list(...))){
+        if(exists(each, envir = parent.frame(), inherits = FALSE)){
+            rm(list = each, pos = parent.frame())
+        }
+    }
+    invisible(NULL)
+}
+remove_if_exists("x")
+1 %into% x
+expect_identical(x, 1)
+
+remove_if_exists("y")
+x = "y"
+
+1 %into% ((x))
+expect_identical(x, "y")
+expect_identical(y, 1)
+
+remove_if_exists("x", "y", "z")
+many_vars = function() c("x", "y", "z")
+list(1,2,3) %into% many_vars()
+expect_identical(x, 1)
+expect_identical(y, 2)
+expect_identical(z, 3)
+
+remove_if_exists("x", "y", "z")
+1:3 %into% many_vars()
+expect_identical(x, 1:3)
+expect_identical(y, 1:3)
+expect_identical(z, 1:3)
+
+remove_if_exists("x", "y", "z")
+
+..[x] = 1
+expect_identical(x, 1)
+
+remove_if_exists("y")
+x = "y"
+
+..[(x)] = 1 
+expect_identical(x, "y")
+expect_identical(y, 1)
+
+
+remove_if_exists("x", "y", "z")
+many_vars = function() c("x", "y", "z")
+..[many_vars()] = list(1,2,3) 
+expect_identical(x, 1)
+expect_identical(y, 2)
+expect_identical(z, 3)
+
+remove_if_exists("x", "y", "z")
+..[many_vars()] = 1:3 
+expect_identical(x, 1:3)
+expect_identical(y, 1:3)
+expect_identical(z, 1:3)
+
+remove_if_exists("x", "y", "z")
+remove_if_exists("x1", "x2", "x3")
+
+..[x1 %to% x3] = list(1:2, letters, TRUE)
+expect_identical(x1, 1:2)
+expect_identical(x2, letters)
+expect_identical(x3, TRUE)
+
 b = 1:5
 b2 = rev(b)
 recode(b, 1~-1, other ~ copy) %into%  b_recoded
@@ -521,7 +603,7 @@ expect_identical(b_recoded, c(-1, 2, 3, 4, 5))
 recode(b, 1~-1, other ~ copy) %into%  "b_recoded_chr"
 expect_identical(b_recoded_chr, c(-1, 2, 3, 4, 5))
 
-recode(dtfrm(b, b2), 1~-1, other ~ copy) %into%  qc(b_recoded)
+recode(list(dtfrm(b, b2)), 1~-1, other ~ copy) %into%  qc(b_recoded)
 rownames(b_recoded) = NULL
 expect_identical(b_recoded, dtfrm(b = c(-1, 2, 3, 4, 5), b2 = rev( c(-1, 2, 3, 4, 5))))
 
@@ -530,6 +612,13 @@ recode(dtfrm(b, b2), 1~10, other ~ copy) %into%  qc(b1_recoded, b2_recoded)
 expect_identical(b1_recoded, c(10, 2, 3, 4, 5))
 expect_identical(b2_recoded, rev( c(10, 2, 3, 4, 5)))
 
+rm(b1_recoded, b2_recoded)
+recode(dtfrm(b, b2), 1~10, other ~ copy) %into%  c(b1_recoded, b2_recoded)
+expect_identical(b1_recoded, c(10, 2, 3, 4, 5))
+expect_identical(b2_recoded, rev( c(10, 2, 3, 4, 5)))
+
+
+
 recode(as.matrix(cbind(b, b2)), 1~11, other ~ copy) %into%  qc(b1_recoded_mat, b2_recoded_mat)
 expect_identical(b1_recoded_mat, c(11, 2, 3, 4, 5))
 expect_identical(b2_recoded_mat, rev( c(11, 2, 3, 4, 5)))
@@ -543,19 +632,19 @@ expect_identical(b3_recoded, c(10, 2, 3, 4, 5))
 expect_identical(b4_recoded, rev( c(10, 2, 3, 4, 5)))
 
 
-recode(dtfrm(b, b1=b, b2), 1~10, other ~ copy) %into%  'v`1:3`'
+recode(dtfrm(b, b1=b, b2), 1~10, other ~ copy) %into%  subst('v`1:3`')
 expect_identical(v1, c(10, 2, 3, 4, 5))
 expect_identical(v2, c(10, 2, 3, 4, 5))
 expect_identical(v3, rev( c(10, 2, 3, 4, 5)))
 i = 1:3
-recode(dtfrm(b, b1=b, b2), 1~10, other ~ copy) %into%  'ww`i`'
+recode(dtfrm(b, b1=b, b2), 1~10, other ~ copy) %into%  subst('ww`i`')
 expect_identical(ww1, c(10, 2, 3, 4, 5))
 expect_identical(ww2, c(10, 2, 3, 4, 5))
 expect_identical(ww3, rev( c(10, 2, 3, 4, 5)))
 
 i = 1:2
 j = 3
-recode(dtfrm(b, b1=b, b2), 1~10, other ~ copy) %into%  c('xx`i`', 'y`j`')
+recode(dtfrm(b, b1=b, b2), 1~10, other ~ copy) %into%  subst(c('xx`i`', 'y`j`'))
 expect_identical(xx1, c(10, 2, 3, 4, 5))
 expect_identical(xx2, c(10, 2, 3, 4, 5))
 expect_identical(y3, rev( c(10, 2, 3, 4, 5)))
@@ -658,4 +747,16 @@ new_iris = modify(new_iris, {
 res_iris[, 1:4] = 
     calc(res_iris, recode (v01 %to% v04, 1 ~ 1, 0 ~ NA))
 
-expect_identical(new_iris, res_iris)
\ No newline at end of file
+expect_identical(new_iris, res_iris)
+
+
+# context("recode factor")
+# 
+# df = data.frame(id = c(1,2,3,4,5),
+#                 Did_you_use_tv=factor(c("tv","","","tv","tv")),
+#                 Did_you_use_internet=factor(c("","","","int","int")))
+# 
+# new_df = df
+# 
+# recode(df[,-1], "" ~ 0, other ~ 1)
+# recode(new_df[,-1]) = c("" ~ 0, other ~ 1)
